# Code written by Theodore Lindsey for his master's thesis.
# Fall 2016
# Liscensed under the GNU Lesser General Public License v3
from os import listdir
from os.path import isfile, join
from utilities import isnum
from math import ceil

# Python version check
from sys import version_info
if version_info < (3,3):
    print("\n\nWARNING: You need to use python version 3.4 or newer.\n\n")
    quit()

# Check for required packages
from pip import get_installed_distributions
# build list of installed packages
installed_packages = [package.project_name for package in get_installed_distributions()]
# list of required_packages
required_packages = ["pandas"]
# check if each required package is installed
for package in required_packages:
    if package not in installed_packages:
        print("\n\nWARNING: You need to install the '{}' package.".format(package))
        print("Try running:\n\tpip install {}\nfrom a privileged terminal".format(package))
        quit()

from IRIM import IRIM

path = "./Inputs"
irim = IRIM()
choice = ""
file = None

# establish default IRIM parameters
minLen = 2
maxLen = 2
ratio = 0.8
minStr = 1
minCov = 1

menu = ["Open file", "View blocks", "Compute rules", "View rules", "Evaluate rules", "Exit"]

while choice != str(1+menu.index("Exit")):
    print("====MENU====")
    # print out the currently open file and path
    if file is not None:
        print("".join(["=" for x in "file=''" + str(join(path, file))]))
        print("file='{}'".format(join(path, file)))
        print("".join(["=" for x in "file=''" + str(join(path, file))]))

    # print out the menu options
    for i, text in enumerate(menu):
        print("{}. {}".format(i+1, text))
    choice = input("> ")
    #############
    # Open file #
    #############
    if choice == str(1+menu.index("Open file")):
        files = [f for f in listdir(path) if isfile(join(path, f))]

        subchoice = ""
        while subchoice not in [str(i) for i in range(1,len(files)+1)]:
            for i, file in enumerate(files):
                print("{}. '{}'".format(i+1, file))
            subchoice = input("> ")
        irim.open(join(path, files[int(subchoice) - 1]))

        minStr = ceil(0.05 * len(irim.lers.records))
    ###############
    # View blocks #
    ###############
    elif choice == str(1+menu.index("View blocks")):
        for attr in irim.lers.blocks:
            for val in irim.lers.blocks[attr]:
                print("[({}, {})] = {}".format(attr, val, irim.lers.blocks[attr][val]))
    #################
    # Compute rules #
    #################
    elif choice == str(1+menu.index("Compute rules")):
        if file is None:
            print("Please first open a file")
        else:
            subchoice = None
            while subchoice not in ["6", ""]:
                print("Data set size: {}".format(len(irim.lers.records)))
                print("Current parameters")
                print("1. Min rule length  : {}".format(minLen))
                print("2. Max rule length  : {}".format(maxLen))
                print("3. Ratio            : {}".format(ratio))
                print("4. Min rule strength: {}".format(minStr))
                print("5. Min rule coverage: {}".format(minCov))
                print("6. accept: Compute rules")
                subchoice = input("> ")
                if subchoice == "1":
                    minLen = int(input("New min rule length: "))
                elif subchoice == "2":
                    maxLen = int(input("New max rule length: "))
                elif subchoice == "3":
                    ratio = float(input("New ratio: "))
                elif subchoice == "4":
                    minStr = int(input("New min rule strength: "))
                elif subchoice == "5":
                    minCov = int(input("New min rule coverage: "))

            irim.computeAllRules(minLen=minLen, maxLen=maxLen, ratio=ratio, minStr=minStr, minCov=minCov)
    ##############
    # View rules #
    ##############
    elif choice == str(1+menu.index("View rules")):
        if irim.rules is None:
            print("Please first compute rules")
        else:
            subchoice = None
            while (not isnum(subchoice)) and (subchoice not in ["a", "A", ""]):
                subchoice = input("Would you like to see all rules (enter 'a')?\nor the top n rules where n=(enter a number)?\n> ")

            if subchoice in ["a", "A", ""]:
                print(irim)
            else:
                stop = int(subchoice)
                for rule in irim[:stop]:
                    print(rule)
    ##################
    # Evaluate rules #
    ##################
    elif choice == str(1+menu.index("Evaluate rules")):
        if irim.rules is None:
            print("Please first compute rules")
        else:
            subchoice = None
            while (not isnum(subchoice)) and (subchoice not in ["a", "A", ""]):
                subchoice = input("Would you like to evaluate all rules (enter 'a')?\nor the top n rules where n=(enter a number)?\n> ")

            if subchoice in ["a", "A", ""]:
                irim.evaluateRules()
            else:
                stop = int(subchoice)
                irim.evaluateRules(stop)
    print()